/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ejb3.test.mdb;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;

import javax.annotation.security.RunAs;
import javax.annotation.Resource;

import org.jboss.annotation.ejb.PoolClass;
import org.jboss.annotation.ejb.TransactionTimeout;
import org.jboss.annotation.security.SecurityDomain;
import org.jboss.logging.Logger;

/**
 * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
 */
@MessageDriven(activationConfig =
        {
        @ActivationConfigProperty(propertyName="destinationType", propertyValue="javax.jms.Queue"),
        @ActivationConfigProperty(propertyName="destination", propertyValue="queue/transactionmdbtest"),
        @ActivationConfigProperty(propertyName="maxSession", propertyValue="1"),
        @ActivationConfigProperty(propertyName="dLQMaxResent", propertyValue="3"),
        @ActivationConfigProperty(propertyName="transactionTimeout", propertyValue="1")
        })
// Wolf: this is NOT supported, use the proper activation config property
//@TransactionTimeout(1)
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@PoolClass(value=org.jboss.ejb3.StrictMaxPool.class, maxSize=1, timeout=10000)
public class TransactionQueueTestMDB implements MessageListener
{
   private static final Logger log = Logger.getLogger(TransactionQueueTestMDB.class);
   
   @Resource 
   private MessageDrivenContext ctx;
   
   public void onMessage(Message recvMsg)
   {
      System.out.println("*** TransactionQueueTestMDB onMessage " + this);
      
      try
      {
    	  Thread.sleep(1000 * 3);
      } 
      catch (InterruptedException e)
      {
    	  e.printStackTrace();
      }
      
      // this is what's supposed to happen
      //ctx.setRollbackOnly();
      
      TestStatusBean.transactionQueueRan++;
   }
}
